{% extends "account/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Sign Up Closed" %}{% endblock head_title %}

{% block inner %}
<h1>{% trans "Sign Up Closed" %}</h1>

<p>{% trans "We are sorry, but the sign up is currently closed." %}</p>
{% endblock %}

