# Generated by Django 2.2.9 on 2020-04-07 21:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0002_email_max_length'),
        ('django_sso_app', '0002_auto_20200307_1035'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailAddress',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('account.emailaddress',),
        ),
    ]
