# Generated by Django 2.2.17 on 2021-04-01 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_sso_app', '0009_auto_20210319_1302'),
    ]

    operations = [
        migrations.CreateModel(
            name='RequestForCredentialsEventListener',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='updated at')),
                ('meta', models.TextField(blank=True, null=True, verbose_name='Meta')),
                ('event_type', models.CharField(default='request_for_credentials', max_length=255)),
                ('confirm_email', models.BooleanField(default=False)),
                ('default_password', models.CharField(blank=True, default=None, max_length=255, null=True)),
                ('is_active_field', models.CharField(default='is_active', max_length=255)),
                ('username_field', models.CharField(default='username', max_length=255)),
                ('email_field', models.CharField(default='email', max_length=255)),
                ('password_field', models.CharField(default='password', max_length=255)),
                ('group_field', models.CharField(default='group', max_length=255)),
                ('sso_id_field', models.CharField(default='sso_id', max_length=255)),
                ('sso_rev_field', models.CharField(default='sso_rev', max_length=255)),
                ('external_id_field', models.CharField(default='external_id', max_length=255)),
                ('ssn_field', models.CharField(default='ssn', max_length=255)),
                ('phone_field', models.CharField(default='phone', max_length=255)),
                ('first_name_field', models.CharField(default='first_name', max_length=255)),
                ('last_name_field', models.CharField(default='last_name', max_length=255)),
                ('alias_field', models.CharField(default='alias', max_length=255)),
                ('description_field', models.CharField(default='description', max_length=255)),
                ('picture_field', models.CharField(default='picture', max_length=255)),
                ('birthdate_field', models.CharField(default='birthdate', max_length=255)),
                ('country_of_birth_field', models.CharField(default='country_of_birth', max_length=255)),
                ('latitude_field', models.CharField(default='position.lat', max_length=255)),
                ('longitude_field', models.CharField(default='position.lon', max_length=255)),
                ('country_field', models.CharField(default='country', max_length=255)),
                ('address_field', models.CharField(default='address', max_length=255)),
                ('language_field', models.CharField(default='language', max_length=255)),
                ('gender_field', models.CharField(default='gender', max_length=255)),
                ('company_name_field', models.CharField(default='company_name', max_length=255)),
                ('expiration_date_field', models.CharField(default='expiration_date', max_length=255)),
                ('device_fingerprint_field', models.CharField(default='device_fingerprint', max_length=255)),
                ('service_url_field', models.CharField(default='service_url', max_length=255)),
            ]
        ),
        migrations.AddField(
            model_name='profile',
            name='created_by_event',
            field=models.BooleanField(default=False, verbose_name='created by event'),
        ),
    ]
