{% extends "account/base.html" %}

{% load i18n %}

{% block title %}{{ flatpage.title }}{% endblock title %}

{% block content %}
<div class="container container-full">
    {{ flatpage.content }}
</div>
{% endblock content %}