# register admin apps

import django_sso_app.core.apps.devices.admin
import django_sso_app.core.apps.groups.admin
import django_sso_app.core.apps.passepartout.admin
import django_sso_app.core.apps.profiles.admin
import django_sso_app.core.apps.services.admin
import django_sso_app.core.apps.users.admin
import django_sso_app.core.apps.events.admin
