# Generated by Django 2.2.17 on 2021-02-22 15:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_sso_app', '0006_delete_emailaddress'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='passepartout',
            unique_together={('token', 'is_active')},
        ),
        migrations.RemoveField(
            model_name='passepartout',
            name='deleted',
        ),
    ]
